/***************************************************************************
                          calltreedlg.cpp  -  description
                             -------------------
    begin                : Fri Jul 18 2003
    copyright            : (C) 2003 by Elad Lahav
    email                : elad_lahav@users.sf.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qpushbutton.h>
#include <klocale.h>
#include "calltreedlg.h"
#include "cscopefrontend.h"

/**
 * Class constructor.
 * @param	pCscope	A Cscope controller to use for querys
 * @param	sFunc	The name of the function that is the root of the tree
 * @param	pParent	The parent widget
 * @param	szName	The widget's name
 */
CallTreeDlg::CallTreeDlg(CscopeFrontend* pCscope, const QString& sFunc,
	QWidget* pParent, const char* szName) :
	CallTreeLayout(pParent, szName),
	m_pCscope(pCscope)
{
	// Make sure the dialog object is deleted when the dialog is closed
	setWFlags(getWFlags() | WDestructiveClose);

	// Close the dialog when the "Close" button is clicked
	connect(m_pCloseButton, SIGNAL(clicked()), this, SLOT(close()));

	// Search for calling functions when an item is first expanded
	connect(m_pTree, SIGNAL(expanded(QListViewItem*)), this,
		SLOT(slotExpandFunc(QListViewItem*)));

	// Open an editor with the matching file and line number, when an item is
	// double-clicked
	connect(m_pTree, SIGNAL(doubleClicked(QListViewItem*)), this,
		SLOT(slotRequestLine(QListViewItem*)));

	// Initialise the tree's columns
	m_pTree->addColumn(i18n("Function"));
	m_pTree->addColumn(i18n("File"));
	m_pTree->addColumn(i18n("Line"));
	m_pTree->addColumn(i18n("Text"));

	// Set the root item
	m_pActiveItem = new QListViewItem(m_pTree, sFunc);
	m_pTree->setOpen(m_pActiveItem, true);

	// Add the immediate callers
	queryCallers(sFunc);
}

/**
 * Class destructor.
 */
CallTreeDlg::~CallTreeDlg()
{
}

/**
 * Adds an entry to the tree, as the child of the active item.
 * Called by a CscopeFrontend object, when a new entry was received in its
 * whole from the Cscope back-end process. The entry contains the data of a
 * function calling the function described by the active item.
 * @param	pToken	The first token in the entry
 */
void CallTreeDlg::dataReady(FrontendToken* pToken)
{
	QString sFile, sFunc, sLine, sText;
	QListViewItem* pItem;

	// Get the file name
	sFile = pToken->getData();
	pToken = pToken->getNext();

	// Get the function name
	sFunc = pToken->getData();
	pToken = pToken->getNext();

	// Get the line number
	sLine = pToken->getData();
	pToken = pToken->getNext();

	// Get the line's text
	sText = pToken->getData();
	pToken = pToken->getNext();

	// Add a list item
	pItem = new QListViewItem(m_pActiveItem, sFunc, sFile, sLine, sText);
	pItem->setExpandable(true);

	// Open the parent item, if not yet opened
	if (!m_pActiveItem->isOpen())
		m_pTree->setOpen(m_pActiveItem, true);
}

/**
 * Disables the expandability feature of an item, if no functions calling it
 * were found.
 * @param	nRecords	Number of records reported by the query
 */
void CallTreeDlg::finished(int nRecords)
{
	if (nRecords == 0)
		m_pActiveItem->setExpandable(false);
}

/**
 * Queries Cscope for functions calling the specified function.
 * @param	sFunc	Function name to query
 */
void CallTreeDlg::queryCallers(const QString& sFunc)
{
	m_pCscope->query(CscopeFrontend::Calling, sFunc, this);
}

/**
 * Looks for calling functions to the function item being expanded.
 * @param	pItem	A tree item being expanded by the user
 */
void CallTreeDlg::slotExpandFunc(QListViewItem* pItem)
{
	// Do not re-query
	if (pItem->childCount() > 0)
		return;
		
	m_pActiveItem = pItem;
	queryCallers(pItem->text(0));
}

/**
 * Emits a signal indicating a certain source file and line number are
 * requested.
 * This slot is connected to the dobuleClicked() signal of the tree. The
 * signal emitted by this slot is used to display an
 * editor page at the requested line.
 * @param	pItem	The list item selected by the user
 */
void CallTreeDlg::slotRequestLine(QListViewItem* pItem)
{
	QString sFilePath, sLine;

	// Get the file name and line number
	sFilePath = pItem->text(1);
	sLine = pItem->text(2);

	// The root item has no associated file
	if (sFilePath.isEmpty())
		return;
		
	// Emit the signal
	emit lineRequested(sFilePath, sLine.toUInt());
}
