/***************************************************************************
                          cscopefrontend.cpp  -  description
                             -------------------
    begin                : Tue May 20 2003
    copyright            : (C) 2003 by Elad Lahav
    email                : elad_lahav@users.sf.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qfileinfo.h>
#include <kconfig.h>
#include <kmessagebox.h>
#include <klocale.h>
#include "cscopefrontend.h"
#include "kscopeconfig.h"
#include "cscopemsgdlg.h"

/**
 * Class constructor.
 * @param	pMsgDlg	A modeless dialog used to display error messages generated
 * 			by the controlled Cscope process
 */
CscopeFrontend::CscopeFrontend(CscopeMsgDlg* pMsgDlg) : 
	Frontend(CSCOPE_ENTRY_SIZE),
	m_state(NotRunning),
	m_bKill(false),
	m_nLines(0),
	m_sErrMsg(""),
	m_pMsgDlg(pMsgDlg)
{
}

/**
 * Class destructor.
 */
CscopeFrontend::~CscopeFrontend()
{
	exit();
}

/**
 * Executes a Cscope process in line mode for the given project.
 * This method starts a new Cscope process, with its standard streams
 * connected to the current process. It uses the "-l" command line option
 * for line mode.
 * Cscope is directed to the given project by setting the name file to
 * "cscope.files" and the cross reference file to "cscope.out" under the
 * project's directory.
 * The full path to the Cscope executable should be set in the "Path" key
 * under the "Cscope" group.
 * @param	sProjPath	The project's directory
 * @param	slOptArgs	Additional command line arguments for Cscope
 * @param	bBuildOnly	Rebuild the database and exit
 * @return	true if successful, false otherwise
 */
bool CscopeFrontend::run(const QString& sProjPath, QStringList& slOptArgs, 
	bool bBuildOnly)
{
	QString sPath;
	QStringList slArgs;
	
	// Restart Cscope if it dies prematurely
	m_bKill = false;
	
	// Make sure the executable exists
	sPath = Config().getCscopePath();
	if (!verify(sPath))
		return false;

	// Set the command line arguments
	slArgs.append(sPath);
	slArgs += slOptArgs;
	if (bBuildOnly) {
		slArgs.append("-b");
		m_pTarget = NULL;
	}
	else {
		slArgs.append("-l");
		if (Config().useVerboseCscope())
			slArgs.append("-v");
	}

	// Run a new process
	if (!Frontend::run(slArgs, sProjPath, m_pTarget))
		return false;

	// Store the current path and options for re-running
	m_sProjPath = sProjPath;
	m_slOptArgs = slOptArgs;
	
	// Initialize stdout parsing
	m_state = NotInitialized;
	m_delim = WSpace;

	// Signal that Cscope is working
	emit stateChanged(false);
	emit progress("Loading project...", 0);

	return true;
}

/**
 * Executes a Cscope query.
 * A query is composed of a numeric type and a query text, which are written
 * to the stndard input of the currently running Cscope process.
 * @param	nType	The type of query to run
 * @param	sText	The query's text
 * @param	pTarget	The target object to accept parsed output data
 */
void CscopeFrontend::query(uint nType, const QString& sText,
	FrontendDataTarget* pTarget)
{
	QString sQuery;

	// Cscope must be ready to accept user commands
	if (m_pProc == NULL || m_state != Ready)
		return;

	// Store the target object for this query
	m_pTarget = pTarget;

	/* Reset the number of lines read */
	m_nLines = 0;
	
	// Write the query to the standard input of the process
	sQuery = QString::number(nType) + sText + '\n';
	m_pProc->writeStdin(sQuery, sQuery.length());

	emit progress("Searching...\n", 0);
}

/**
 * Rebuilds the symbol database of the current project.
 */
void CscopeFrontend::rebuild()
{
	// If Cscope is not running, run it now (may happen if the project did
	// not contain any files when Cscope was first launched)
	if (!isRunning()) {
		run(m_sProjPath, m_slOptArgs);
		return;
	}

	// Cscope must be ready to accept user commands
	if (m_state != Ready)
		return;

	// Enter the rebuild command
	m_pProc->writeStdin("r\n", 2);
	m_state = BuildPrompt;
	emit stateChanged(false);
}

/**
 * Stops a running Cscope process.
 * Reimplemented from Frontend, to reset the current parsing state after the
 * process has terminated.
 */
void CscopeFrontend::exit()
{
	m_bKill = true;
	Frontend::exit();
	m_state = NotInitialized;
}

/**
 * Tests that the given file path leads to an executable.
 * @param	sPath	The path to check
 * @return	true if the file in the given path exists and has executable
 *			permissions, false otherwise
 */
bool CscopeFrontend::verify(const QString& sPath)
{
	QFileInfo fi(sPath);

	if (!fi.exists() || !fi.isFile() || !fi.isExecutable() ||
		fi.fileName() != "cscope") {
		KMessageBox::error(0, i18n("Cscope cannot be found in the given "
			"path"));
		return false;
	}

	return true;
}

/**
 * Stops a Cscope action.
 */
void CscopeFrontend::slotCancel()
{
	// Nothing to do if Cscope is not running
	if (!isRunning())
		return;

	// Do not send interrupt signal if Cscope is idle
	if (m_state == NotInitialized || m_state == Ready)
		return;

	m_pProc->kill(SIGINT);
	m_state = NotInitialized;
	m_delim = All; // Ignore everything but a prompt (">>") sign
}

/**
 * Parses the output of a Cscope process.
 * Implements a state machine, where states correspond to the output of the
 * controlled Cscope process.
 * @param	sToken	The current token read (the token delimiter is determined
 *					by the current state)
 * @return	A value indicating the way this token should be treated: dropped,
 *			added to the token queue, or finishes a new record
 */
Frontend::ParseResult CscopeFrontend::parseStdout(QString& sToken,
	ParserDelim /* ignored */)
{
	ParserState statePrev;
	int nFiles, nTotalFiles;
	ParseResult result = DiscardToken;

	// Store the last state
	statePrev = m_state;

	// Handle the token according to the current state
	switch (m_state) {
	case NotInitialized:
		if (sToken == ">>") {
			// If a prompt is given, Cscope is ready to accept queries
			m_state = Ready;
		}
		else if (sToken == "Building") {
			// "Building" is received if the cross reference file is
			// being built
			m_state = Building;
			m_delim = Newline;
		}

		result = DiscardToken;
		break;

	case Building:
		// Try to get building progress
		if (sscanf(sToken.latin1(), "Building symbol "
			"database %d of %d", &nFiles, &nTotalFiles) == 2) {
			emit progress(i18n("Building cross reference..."),
				(nFiles * 100) / nTotalFiles);
		}

		// Wait for another progress line or the "ready" symbol
		m_state = BuildPrompt;
		m_delim = WSpace;

		result = DiscardToken;
		break;

	case BuildPrompt:
		// A single angle bracket is the prefix of a progress indication,
		// while double brackets is Cscope's prompt for a new query
		if (sToken == ">") {
			m_state = Building;
			m_delim = Newline;
		}
		else if (sToken == ">>") {
			m_state = Ready;
		}

		result = DiscardToken;
		break;

	case Ready:
	case SearchPrompt:
		// Check for more search progress, or the end of the search,
		// designated by a line in the format of "cscope: X lines"
		if (sToken == ">") {
			m_state = Search;
			m_delim = Newline;
		}
		else if (sToken == "cscope:") {
			m_state = QueryEnded;
			m_delim = Newline;
		}

		result = DiscardToken;
		break;

	case Search:
		// Try to get the search progress value (ignore other messages)
		if (sscanf(sToken.latin1(), "Search %d of %d", &nFiles,	&nTotalFiles)
			== 2) {
			emit progress(i18n("Searching..."), (nFiles * 100) / nTotalFiles);
		}

		m_state = SearchPrompt;
		m_delim = WSpace;
		result = DiscardToken;
		break;

	case QueryEnded:
		// Get the number of records found, or reset the state (if the
		// "cscope:" token is followed by something else than the total
		// records indication, which should not happen)
		if (sscanf(sToken.latin1(), "%d lines", &m_nLines) == 1) {
			// Notify the taget on the number of total lines
			if (m_pTarget && m_nLines == 0)
				m_pTarget->finished(0);

			m_state = File;
			m_delim = WSpace;
			emit progress(i18n("Query finished, please wait"), 100);
		}
		else {
			m_state = NotInitialized;
			m_delim = WSpace;
		}

		result = DiscardToken;
		break;

	case File:
		// A Cscope prompt in this state signifies the end of the records
		// found in the current state
		if (sToken == ">>") {
			if (m_pTarget && m_nLines > 0)
				m_pTarget->finished(m_nLines);
			
			m_state = Ready;
			result = DiscardToken;
			break;
		}

		// Treat the token as the name of the file in this record
		m_state = Func;
		result = AcceptToken;
		break;

	case Func:
		// Treat the token as the name of the function in this record
		m_state = Line;
		result = AcceptToken;
		break;

	case Line:
		// Treat the token as the line number in this record
		m_state = LineText;
		m_delim = Newline;
		result = AcceptToken;
		break;

	case LineText:
		// Treat the token as the text of this record, and report a new
		// record
		m_state = File;
		m_delim = WSpace;
		result = EntryReady;
		break;

	default:
		// Do nothing (prevents a compilation warning for unused enum values)
		break;
	}

	// Notify if the readiness status has changed
	if ((m_state == Ready) && (statePrev != Ready))
		emit stateChanged(true);
	else if ((m_state != Ready) && (statePrev == Ready))
		emit stateChanged(false);

	return result;
}

/**
 * Handles Cscope messages sent to the standard error stream.
 * @param	sText	The error message text
 */
void CscopeFrontend::parseStderr(const QString& sText)
{
	// Wait for a complete line to arrive
	m_sErrMsg += sText;
	if (!sText.endsWith("\n"))
		return;
	
	// Do not re-run Cscope if exited due to lack of project files
	if (m_sErrMsg.contains("no source files found"))
		m_bKill = true;
	
	// Do not re-run Cscope if cross-reference file could not be found
	if (m_sErrMsg.contains("cscope: cannot open file"))
		m_bKill = true;
	
	// Display the error message
	if (m_pMsgDlg != NULL)
		m_pMsgDlg->addText(m_sErrMsg);
	
	// Line displayed, reset the text accumulator
	m_sErrMsg = "";
}

/**
 * Called when Cscope exits.
 * If Cscope has died prematurely, it is restarted.
 */
void CscopeFrontend::finalize()
{
	// Reset the current state
	m_state = NotRunning;

	// Restart Cscope, unless specifically killed
	if (!m_bKill)
		run(m_sProjPath, m_slOptArgs);
}
