/***************************************************************************
                          cscopefrontend.h  -  description
                             -------------------
    begin                : Tue May 20 2003
    copyright            : (C) 2003 by Elad Lahav
    email                : elad_lahav@users.sf.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CSCOPEFRONTEND_H
#define CSCOPEFRONTEND_H

#include <qstringlist.h>
#include "frontend.h"

class CscopeMsgDlg;

#define CSCOPE_ENTRY_SIZE 4

/**
 * Controls a Cscope process for the current project.
 * This class creates a Cscope process, using the project's files for
 * configuration. Once the process is running, KScope uses the query() method
 * to initiate Cscope queries on the project's files. The queries' output is
 * parsed into a set of records, each consisting of the following fields:
 * - File name
 * - Function name
 * - Line number
 * - The line's text
 * These records are used to display the output in different windows, such as
 * QueryWidget and CallTreeDlg.
 * @author Elad Lahav
 */

class CscopeFrontend : public Frontend
{
	Q_OBJECT

public:
	CscopeFrontend(CscopeMsgDlg* pMsgDlg = NULL);
	~CscopeFrontend();

	/**
	 * The available Cscope query types.
	 */
	enum QueryType { Reference = 0, Definition = 1, Called = 2, Calling = 3,
		Text = 4, Pattern = 6, FileName = 7, Including = 8, None = 9 };

	bool run(const QString&, QStringList&, bool bBuildOnly = false);
	void query(uint, const QString&, FrontendDataTarget*);
	void rebuild();
	virtual void exit();
	virtual bool verify(const QString&);
	
public slots:
	void slotCancel();

signals:
	/**
	 * Emitted when the Cscope process changes its state.
	 * Possible states are 'ready', which means that a new command can be 
	 * sent to the process, and 'working', which means the process is 
	 * currently busy.
	 * @param	bReady	true if the Cscope process is ready, false otherwise
	 */
	void stateChanged(bool bReady);
	
	/**
	 * Reports the current stage of an ongoing work performed by the Cscope
	 * process (while building the database or querying it.)
	 * @param	sMsg	A text message describing the work being done
	 * @param	nProg	The percentage of work already done
	 */
	void progress(const QString& sMsg, int nProg);

protected:
	virtual ParseResult parseStdout(QString&, ParserDelim);
	virtual void parseStderr(const QString&);
	virtual void finalize();

private:
	/**
	 * The possible states of the parser state machine.
	 */
	enum ParserState { NotRunning = 0, NotInitialized, Building, BuildPrompt,
		Ready, SearchPrompt, Search, QueryEnded, File, Func, Line,
		LineText };
	
	/** The current state of the parser state machine. */
	ParserState m_state;
	
	/** The full path of the directory holding the project files. */
	QString m_sProjPath;
	
	/** Additional command-line arguments to pass to Cscope. */
	QStringList m_slOptArgs;
	
	/** Determines if Cscope should be re-run in case the process exits 
		prematurely. Setting this variable to 'true' means we should let
		Cscope die and not restart the process. */
	bool m_bKill;
	
	/** Holds the number of lines in the result of a Cscope query. */
	int m_nLines;
	
	/** Accumulates text sent by Cscope to the standard error stream. */
	QString m_sErrMsg;

	/** A dialogue to display Cscope error messages. */
	CscopeMsgDlg* m_pMsgDlg;
};

#endif
