/***************************************************************************
                          newprojectdlg.cpp  -  description
                             -------------------
    begin                : Mon Jun 9 2003
    copyright            : (C) 2003 by Elad Lahav
    email                : elad_lahav@users.sf.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qregexp.h>
#include <qpushbutton.h>
#include <kfiledialog.h>
#include <kmessagebox.h>
#include <klocale.h>
#include "newprojectdlg.h"

/**
 * Class constructor.
 * @param	bNewProj	true to create a new project dialog, false to display
 *						the properties of an existing project
 * @param	pParent		The parent widget
 * @param	szName		The widget's name
 */
NewProjectDlg::NewProjectDlg(bool bNewProj, QWidget* pParent, 
	const char* szName) :
	NewProjectLayout(pParent, szName)
{
	connect(m_pCreateButton, SIGNAL(clicked()), this, SLOT(accept()));
	connect(m_pCancelButton, SIGNAL(clicked()), this, SLOT(reject()));

	// Add the file type enetered in the type line edit
	connect(m_pAddButton, SIGNAL(clicked()), this, SLOT(slotAddType()));

	// Remove the selected file type from the list
	connect(m_pRemoveButton, SIGNAL(clicked()), this,
		SLOT(slotRemoveType()));
	
	// Perform actions specific to the type of dialog (new project or
	// project properties)
	if (bNewProj) {
		// Add the default file types
		m_pTypesList->insertItem("*.c");
		m_pTypesList->insertItem("*.h");

		// Search for a project's directory when the "Find..." button is 
		// clicked
		connect(m_pFindButton, SIGNAL(clicked()), this, 
			SLOT(slotFindProject()));
	}
	else {
		// Give appropriate titles to the dialog and the accept button
		setCaption(i18n("Project Properties"));
		m_pCreateButton->setText(i18n("OK"));
		
		// Disable the non-relevant widgets
		m_pNameEdit->setEnabled(false);
		m_pPathEdit->setEnabled(false);
		m_pFindButton->setEnabled(false);
	}
}

/**
 * Class destructor.
 */
NewProjectDlg::~NewProjectDlg()
{
}

/**
 * Ends the dialog after the user has clicked the "OK" button.
 */
void NewProjectDlg::accept()
{
	int i, nCount;
	
	// Fill the string list with all file types
	nCount = (int)m_pTypesList->count();
	for (i = 0; i < nCount; i++)
		m_slTypes.append(m_pTypesList->text(i));

	// Close the dialog
	QDialog::accept();
}

/**
 * Enables the user to search for a directory in which the project will be
 * created.
 * This slot is called when the "Find.." button is clicked.
 */
void NewProjectDlg::slotFindProject()
{
	QString sDir;

	// Prompt the user for a directory
	sDir = KFileDialog::getExistingDirectory();
	if (!sDir.isEmpty())
		m_pPathEdit->setText(sDir);
}

/**
 * Adds the file type string in the type line edit to the list of types.
 * This slot is called when the "Add.." button is clicked.
 */
void NewProjectDlg::slotAddType()
{
	QString sType;

	// Get the type string
	sType = m_pTypesEdit->text();
	sType.stripWhiteSpace();
	if (sType.isEmpty())
		return;

	// Validate the type string
	QRegExp reg("[ \\t\\n\\|\\\\\\/]");
	if (sType.contains(reg)) {
		KMessageBox::error(0, i18n("This is not a valid file type!"));
		return;
	}

	// Add the file type to the list
	m_pTypesList->insertItem(sType);
	m_pTypesEdit->clear();
}

/**
 * Removes the selected item from the list of file types.
 * This slot is called when the "Remove" button is clicked.
 */
void NewProjectDlg::slotRemoveType()
{
	int nItem;
		
	// Verify an item is selected
	nItem = m_pTypesList->currentItem();
	if (nItem == -1)
		return;

	// Remove the selected item
	m_pTypesList->removeItem(nItem);
}

/**
 * Retrieves the text entered by the user in the dialog's "Project Name" edit
 * box.
 * @return	The name of the new project
 */
QString NewProjectDlg::getName()
{
	return m_pNameEdit->text();
}

/**
 * Retrieves the text entered by the user in the dialog's "Project Path" edit
 * box.
 * Note that the chosen path will be the parent of the new project's
 * directory, created under it using the project's name.
 * @return	The full path of the parent directory for the new project
 */
QString NewProjectDlg::getPath()
{
	return m_pPathEdit->text();
}

/**
 * Returns the list of regular expressions corresponding to file types for
 * the project.
 * These types will be considered when scanning directories for files to add
 * to the project.
 * @return	The list of file types associated with the project.
 */
const QStringList& NewProjectDlg::getTypes()
{
	return m_slTypes;
}

/**
 * Determines if the user has checked the "Kernel Project" option.
 * @return	true if the project is a kernel project, false otherwise.
 */
bool NewProjectDlg::isKernel()
{
	return m_pKernelCheck->isChecked();
}

/**
 * Determines if the user has checked the "Inverted Index" option.
 * @return	true if an inverted index should be created for the project, false
 *			otherwise.
 */
bool NewProjectDlg::useInvIndex()
{
	return m_pInvCheck->isChecked();
}

/**
 * Configures the dialog's widget to display the properties of the current
 * project.
 * @param	sName		The name of the current project
 * @param	sPath		The full path of the project's parent directory
 * @param	slTypes		The file types included in the project
 * @param	bKernel		The value of the "Kernel Project" option
 * @param	bInvIndex	The value of the "Inverted Index" option
 */
void NewProjectDlg::setProperties(const QString& sName, const QString& sPath,
	const QStringList& slTypes, bool bKernel, bool bInvIndex)
{
	QStringList::ConstIterator itr;
		
	// Set values for current project
	m_pNameEdit->setText(sName);
	m_pPathEdit->setText(sPath);
	m_pKernelCheck->setChecked(bKernel);
	m_pInvCheck->setChecked(bInvIndex);
	
	// Add type strings to the types list box
	for (itr = slTypes.begin(); itr != slTypes.end(); ++itr)
		m_pTypesList->insertItem(*itr);
}
