/***************************************************************************
                          positionhistory.cpp
                             -------------------
    begin                : Sat April 3 2004
    copyright            : (C) 2004 by Elad Lahav
    email                : elad_lahav@users.sf.net
 ***************************************************************************/

#include "positionhistory.h"
#include "kscopeconfig.h"

/**
 * Class constructor.
 */
PositionHistory::PositionHistory() : 
	m_bAccept(true),
	m_list(),
	m_itr(m_list.begin())
{
}

/**
 * Class destructor.
 */
PositionHistory::~PositionHistory()
{
}

/**
 * Toggles the accept mode of the history list.
 * @param	bAccept	true to accept new entries, false otherwise
 */
void PositionHistory::accept(bool bAccept)
{
	m_bAccept = bAccept;
}

/**
 * Prepends a file position to the hidtory list.
 * @param	sFilePath	The full path of the file
 * @param	nLine		The line number
 */
void PositionHistory::add(const QString& sFilePath, uint nLine)
{
	// Must be ready to accept new entries
	if (!m_bAccept)
		return;

	// Validate entry
	if (sFilePath.isEmpty() || nLine == 0)
		return;
	
	if (!m_list.empty()) {
		// Do not add the same entry twice
		if (((*m_itr).m_sFilePath == sFilePath) &&
			((*m_itr).m_nLine == nLine)) {
			return;
		}
		
		// If adding an entry while the marker is not positioned at the top of
		// the list, remove all entries above the marker
		if (m_itr != m_list.begin())
			m_list.erase(m_list.begin(), m_itr);
	 	
		// Make room for the new entry, if required
		if (m_list.size() == Config().getMaxPosHistory())
			m_list.erase(m_list.fromLast());
	}
	
	// Add the new enrry at the top of the list
	m_list.prepend(FilePosition(sFilePath, nLine));
	m_itr = m_list.begin();
}

/**
 * Retrieves the history item marked as 'current'.
 * @param	sFilePath	Holds the full path specified in the current item
 *						(upon successful return)
 * @param	nLine		Holds the line number specified in the current item
 *						(upon successful return)
 * @return	true if successful, false if the position history list is empty
 */
bool PositionHistory::current(QString& sFilePath, uint& nLine)
{
	// Abort if the list is empty
	if (m_list.empty())
		return false;
	
	// Set requested file path and line number to match current item
	sFilePath = (*m_itr).m_sFilePath;
	nLine = (*m_itr).m_nLine;
	
	return true;
}

/**
 * Moves to a more recent item in the history list, if such an item exists.
 * @param	sFilePath	Holds the full path to the file in the new entry, if
 *						move was successful
 * @param	nLine		Holds the line number in the new entry, if move was 
 *						successful
 * @return	true if the list iterator was repositioned, false if the current
 *			entry was the latest in the list
 */
bool PositionHistory::next(QString& sFilePath, uint& nLine)
{
	// Abort if already at the top of the list
	if (m_itr == m_list.begin())
		return false;
	
	// Change list iterator position
	m_itr--;
	
	// Set requested file path and line number to match new item
	sFilePath = (*m_itr).m_sFilePath;
	nLine = (*m_itr).m_nLine;
	
	return true;
}

/**
 * Moves to an earlier item in the history list, if such an item exists.
 * @param	sFilePath	Holds the full path to the file in the new entry, if
 *						move was successful
 * @param	nLine		Holds the line number in the new entry, if move was 
 *						successful
 * @return	true if the list iterator was repositioned, false if the current
 *			entry was the earliest in the list
 */
bool PositionHistory::prev(QString& sFilePath, uint& nLine)
{
	// Abort if already at the top of the list, or list is empty
	if (m_itr == m_list.fromLast())
		return false;
	
	// Change list iterator position
	m_itr++;
	
	// Set requested file path and line number to match new item
	sFilePath = (*m_itr).m_sFilePath;
	nLine = (*m_itr).m_nLine;
	
	return true;
}

/**
 * Positions the history list iterator at the given item.
 * @param	nIndex	The 0-based index of the item to which the iterator should
 *					point
 */
void PositionHistory::moveTo(int nIndex)
{
	QValueList<FilePosition>::Iterator itr;
	
	if ((itr = m_list.at(nIndex)) != m_list.end())
		m_itr = itr;
}

/**
 * Deletes all items in the history list.
 */
void PositionHistory::clear()
{
	m_list.erase(m_list.begin(), m_list.end());
}

/**
 * Passes the file path and line number of each item in the position list as
 * parameters to the add() method of a PositionListTarget object.
 * @param	pList	The object item to fill
 * @return	true if successful, false if the position list is empty
 */
bool PositionHistory::fillList(PositionListTarget* pList)
{
	QValueList<FilePosition>::Iterator itr;
	int nIndex;
	
	// Abort if no history was recorded
	if (m_list.empty())
		return false;
	
	// Add the list items
	for (itr = m_list.begin(), nIndex = 0; itr != m_list.end(); ++itr, 
		nIndex++) {
		pList->addItem((*itr).m_sFilePath, (*itr).m_nLine, nIndex, 
			itr == m_itr);
	}
	
	return true;
}

/**
 * Provides global access to a single PositionHistory item.
 * @return	Reference to the global PositionHistory object
 */
PositionHistory& PosHistory()
{
	static PositionHistory ph;
	return ph;
}
