/***************************************************************************
                          projectmanager.cpp  -  description
                             -------------------
    begin                : Sat May 10 2003
    copyright            : (C) 2003 by Elad Lahav
    email                : elad_lahav@users.sf.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <unistd.h>
#include <kmessagebox.h>
#include <klocale.h>
#include "projectmanager.h"
#include "kscopeconfig.h"

#define CONFIG_VER 2

/**
 * Class constructor.
 */
ProjectManager::ProjectManager() :
	m_pFiles(NULL),
	m_pConf(NULL),
	m_bOpen(false),
	m_bTempProj(false)
{
}

/**
 * Class destructor.
 */
ProjectManager::~ProjectManager()
{
	close();
}

/**
 * Creates a project's directory, and associates this directory with the
 * current object. This directory is created under the given path, and using
 * the project's name (which, thus, has to be a legal file name). 
 * Note: this function attempts to create a new directory, so the given path
 * and name must not lead to an existing one.
 * @param	sName		The project's name
 * @param	sPath		The parent directory under which to create the 
 *						project's directory
 * @param	bKernel		true to create a kernel project (a project that does
 *						not rely on /usr/include), false otherwise
 * @param	bInvIndex	true to create an inverted index, false otherwise
 * @param	slTypes		List of filters for source files
 * @return	true if successful, false otherwise
 */
bool ProjectManager::create(const QString& sName, const QString& sPath, 
	bool bKernel, bool bInvIndex, const QStringList& slTypes)
{
	QFile* pFilesFile;
	KConfig* pConf;
	
	// If the project is open, close it first
	if (m_bOpen)
		close();

	// Make sure the directory doesn't exist
	QDir dir(sPath);
	if (dir.exists(sName)) {
		KMessageBox::error(0, i18n("Cannot create a project inside an "
			"existing directory"));
		return false;
	}

	// Try to create the projcet's directory
	if (!dir.mkdir(sName, false) || !dir.cd(sName, false)) {
		KMessageBox::error(0, i18n("Failed to create the project's "
			"directory"));
		return false;
	}
	
	// Prepare the project's files
	initConfig(dir.absPath(), &pFilesFile, &pConf);

	// Write the configuration file version
	pConf->setGroup("");
	pConf->writeEntry("Version", CONFIG_VER);
	
	// Write project properties in the configuration file
	pConf->setGroup("Project");
	pConf->writeEntry("Name", sName);
	pConf->writeEntry("Kernel", bKernel);
	pConf->writeEntry("InvIndex", bInvIndex);
	pConf->writeEntry("FileTypes", slTypes.join(" "));

	// Flush the config file data, so the project is created even if KScope
	// crashes...
	pConf->sync();
	delete pConf;

	// Create the "files" file
	if (!pFilesFile->open(IO_WriteOnly)) {
		delete pFilesFile;
		KMessageBox::error(0, i18n("Failed to open the \"files\" file"));
		return false;
	}

	pFilesFile->close();
	delete pFilesFile;
	
	return true;
}

/**
 * Opens a project and makes it the current one.
 * @param	sPath	The directory containing the project's files
 * @return	true if successful, false otherwise
 */
bool ProjectManager::open(const QString& sPath)
{
	QString sConfFile;
	
	// Associate the object with the project directory
	m_dir.setPath(sPath);
	if (!m_dir.exists()) {
		KMessageBox::error(0, i18n("Project directory does not exist"));
		return false;
	}

	// Open the configuration files
	initConfig(sPath, &m_pFiles, &m_pConf);

	// Verify the configuration file's version is compatible
	m_pConf->setGroup("");
	if (m_pConf->readUnsignedNumEntry("Version", 0) != CONFIG_VER) {
		KMessageBox::error(0, i18n("Your project is not compatible with this "
		"version of KScope.\nPlease re-create the project."));
		return false;
	}
	
	// Get the project name
	m_pConf->setGroup("Project");
	m_sName = m_pConf->readEntry("Name");
	if (m_sName == QString::null) {
		KMessageBox::error(0, i18n("Cannot read project name"));
		return false;
	}

	// Add to the list of recently opened projects
	Config().addRecentProject(sPath);
		
	// Indicate that a project has been opened
	m_bOpen = true;
	m_bTempProj = false;
	
	return true;
}

/**
 * Opens a Cscope.out file as a temporary project.
 * @param	sFilePath	The full path of the Cscope.out file
 * @return	true if successful, false otherwise
 */
bool ProjectManager::openCscopeOut(const QString& sFilePath)
{
	QFileInfo fi(sFilePath);
	
	// Make sure the file exists
	if (!fi.exists())
		return false;
	
	// Set the project's directory
	m_dir = fi.dirPath(true);
	
	// Set the name of the project to be the full path of the file
	m_sName = fi.absFilePath();
	
	// Indicate that a project has been opened
	m_bOpen = true;
	m_bTempProj = true;
	
	return true;
}

/**
 * Performs clean-up on the project's variables, and detaches the associated
 * directory.
 */
void ProjectManager::close()
{
	if (!m_bTempProj) {
		m_dir = QDir();
		delete m_pFiles;
		m_pFiles = NULL;
		delete m_pConf;
		m_pConf = NULL;
	}
	
	m_bOpen = false;
}

/**
 * Adds a set of files to the current project.
 * @param	slFiles	A list of file names to add
 * @return	The total number of files added to the project
 */
int ProjectManager::addFiles(const QStringList& slFiles)
{
	QStringList::const_iterator itr;
	int nFiles = 0;

	// Cannot add files if no project is open, or the project is temporary
	if (!m_bOpen || m_bTempProj)
		return 0;

	// Open the source files file, to which the file paths will be added
	if (!m_pFiles->open(IO_WriteOnly | IO_Append))
		return 0;

	// Add each file to the project
	QTextStream str(m_pFiles);
	for (itr = slFiles.begin(); itr != slFiles.end(); ++itr) {
		str << *itr << "\n";
		nFiles++;
	}

	m_pFiles->close();
	emit filesAdded(slFiles);
	
	return nFiles;
}

/**
 * Fills a list object with all files in the project.
 * List items are created by reading and parsing all file name entries from
 * the project's 'cscope.files' file.
 * Note that the file may contain option lines, beginning with a dash. These
 * should be ignored.
 * @param	pList	Pointer to the object to fill
 */
void ProjectManager::fillList(FileListTarget* pList)
{
	QString sFilePath;
	
	// Must have an open persistent project
	if (!m_bOpen || m_bTempProj)
		return;
	
	// Open the 'cscope.files' file
	if (!m_pFiles->open(IO_ReadOnly))
		return;

	// Read all file names from the file
	QTextStream str(m_pFiles);
	while ((sFilePath = str.readLine()) != QString::null) {
		// Skip option lines
		if (sFilePath.at(0) == '-')
			continue;

		// Set the new list item
		pList->addItem(sFilePath);
	}

	m_pFiles->close();
}

/**
 * Writes all file entries in a list view widget to the project's
 * 'cscope.files' file (replacing current file contents.)
 * @param	pList	Pointer to the object from which to take the new entries
 */
void ProjectManager::writeList(FileListSource* pList)
{
	QString sFilePath;
	
	// Must have an open persistent project
	if (!m_bOpen || m_bTempProj)
		return;
	
	// Open the 'cscope.files' file
	if (!m_pFiles->open(IO_WriteOnly))
		return;

	QTextStream str(m_pFiles);

	// Write all file names
	if (pList->firstItem(sFilePath)) {
		do {
			str << sFilePath << "\n";
		} while (pList->nextItem(sFilePath));
	}

	m_pFiles->close();
	emit fileListChanged();
}

/**
 * Determines whether the project includes any files.
 * Reads the 'cscope.files' file and looks for any file names in it. If none
 * is present, then the project is considered empty.
 * Note that the file may contain option lines, beginning with a dash. These
 * should be ignored.
 * @return	true if no files are included in the project, false otherwise
 */
bool ProjectManager::isEmpty()
{
	QString sPath, sFileName;
	bool bResult = true;
	
	// Must have an active project
	if (!m_bOpen)
		return true;

	// Assume there are some files if using only a cscope.out file
	if (m_bTempProj)
		return false;
	
	// Open the 'cscope.files' file
	if (!m_pFiles->open(IO_ReadOnly))
		return true;

	// Find at least one file name entry in the file
	QTextStream str(m_pFiles);
	while ((sPath = str.readLine()) != QString::null) {
		if (sPath.at(0) != '-') {
			bResult = false;
			break;
		}
	}

	m_pFiles->close();
	return bResult;
}

/**
 * Returns a semi-colon separated list of the file types included in the
 * current project.
 */
QString ProjectManager::getFileTypes()
{
	QString sTypes;
	
	// Must have an open persistent project
	if (!m_bOpen || m_bTempProj)
		return QString::null;

	m_pConf->setGroup("Project");
	return m_pConf->readEntry("FileTypes");
}

/**
 * Sets the list of source file types that should be included in this project.
 * The list is used as a filter when adding files and scanning directories for
 * source files.
 * @param	slTypes	List of filters for source files
 */
void ProjectManager::setFileTypes(const QStringList& slTypes)
{
	// Must have an open persistent project
	if (!m_bOpen || m_bTempProj)

	m_pConf->setGroup("Project");
	m_pConf->writeEntry("FileTypes", slTypes.join(" "));
}


/**
 * Reads the project's options from the configuration file.
 * @param	bKernel		Holds the value of the "Kernel Project" option, upon 
 *						return
 * @param	bInvIndex	Holds the value of the "Inverted Index" option, upon 
 *						return
 */
void ProjectManager::getOptions(bool& bKernel, bool& bInvIndex)
{
	// Must have an open persistent project
	if (!m_bOpen || m_bTempProj)
		return;

	// Write the options line, if required
	m_pConf->setGroup("Project");
	bKernel = m_pConf->readBoolEntry("Kernel");
	bInvIndex = m_pConf->readBoolEntry("InvIndex");
}

/**
 * Creates a list of command line arguments required for running Cscope on
 * the current project.
 * @param	slArgs	The list to fill
 */
void ProjectManager::getOptions(QStringList& slArgs)
{
	// Do nothing if there is no open project
	if (!m_bOpen)
		return;

	// Parameters for temporary projects: do not update the cross-reference
	// file
	if (m_bTempProj) {
		slArgs.append("-d");
		return;
	}
	
	// Write the options line, if required
	m_pConf->setGroup("Project");
	if (m_pConf->readBoolEntry("Kernel"))
		slArgs.append("-k");
	
	if (m_pConf->readBoolEntry("InvIndex"))
		slArgs.append("-q");
}

/**
 * Sets project options that affect the way Cscope is run on the project's
 * files.
 * @param	bKernel		true to create a kernel project (a project that does
 *						not rely on /usr/include), false otherwise
 * @param	bInvIndex	true to create an inverted index, false otherwise
 */
void ProjectManager::setOptions(bool bKernel, bool bInvIndex)
{
	// Must have an open persistent project
	if (!m_bOpen || m_bTempProj)
		return;

	// Set new project options
	m_pConf->setGroup("Project");
	m_pConf->writeEntry("Kernel", bKernel);
	m_pConf->writeEntry("InvIndex", bInvIndex);		
}

/**
 * Reads the paths of the files that were open when the project was closed,
 * and fills a list used to restore the previous session.
 * @param	slOpenFiles	The list to fill
 */
void ProjectManager::getOpenFiles(QStringList& slOpenFiles)
{
	// Must have an open persistent project
	if (!m_bOpen || m_bTempProj)
		return;

	// Read the file paths
	m_pConf->setGroup("Session");
	slOpenFiles = m_pConf->readListEntry("OpenFiles");
}

/**
 * Writes a list of open files to the project's configuration file.
 * The method is called before a project is closed, so these file paths can
 * be read when the project is restored.
 * @param	slOpenFiles	The list of file paths to write
 */
void ProjectManager::setOpenFiles(QStringList& slOpenFiles)
{
	// Must have an open persistent project
	if (!m_bOpen || m_bTempProj)
		return;

	// Write the file paths
	m_pConf->setGroup("Session");
	m_pConf->writeEntry("OpenFiles", slOpenFiles);
}

/**
 * Reads the names of the files corresponding to queries that were locked
 * when the project was closed, and fills a list used to restore the
 * previous session.
 * @param	slQueryFiles	The list to fill
 */
void ProjectManager::getQueryFiles(QStringList& slQueryFiles)
{
	// Must have an open persistent project
	if (!m_bOpen || m_bTempProj)
		return;

	// Read the file paths
	m_pConf->setGroup("Session");
	slQueryFiles = m_pConf->readListEntry("QueryFiles");
}

/**
 * Writes a list of query file names to the project's configuration file.
 * The method is called before a project is closed, so these file names can
 * be read when the project is restored.
 * @param	slQueryFiles	The list of file names to write
 */
void ProjectManager::setQueryFiles(QStringList& slQueryFiles)
{
	// Must have an open persistent project
	if (!m_bOpen || m_bTempProj)
		return;

	// Write the file names
	m_pConf->setGroup("Session");
	m_pConf->writeEntry("QueryFiles", slQueryFiles);
}

/**
 * Reads the value of the project's root directory from the project's
 * configuration file.
 * @return	The root directory of the active project
 */
QString ProjectManager::getRoot()
{
	// Empty root if no project is currently open
	if (!m_bOpen)
		return "";
	
	// Return the file-system root if using a temporary project
	if (m_bTempProj)
		return "/";

	m_pConf->setGroup("Project");
	return m_pConf->readEntry("RootPath", "/");
}

/**
 * Sets a new root directory for a project.
 * @param	sRoot	The full path of the new root directory
 */
void ProjectManager::setRoot(const QString& sRoot)
{
	// Do nothing if there is no open project, or using a temporary project
	if (!m_bOpen || m_bTempProj)
		return;
	
	m_pConf->setGroup("Project");
	m_pConf->writeEntry("RootPath", sRoot);
}

/**
 * Creates the "cscope.files" and "cscope.proj" file objects for the given
 * project.
 * @param	sPath		The project directory, in which the files reside
 * @param	ppFilesFile	Holds the new "cscope.files" object, upon return
 * @param	ppConf		Holds the new "cscope.proj" object, upon return
 */
void ProjectManager::initConfig(const QString& sPath, QFile** ppFilesFile,
	KConfig** ppConf)
{
	QString sFilesFile, sConfFile;

	// Attach to the source files file
	sFilesFile = sPath + "/cscope.files";
	*ppFilesFile = new QFile(sFilesFile);
	
	// Open the configuraton file
	sConfFile = sPath + "/cscope.proj";	
	*ppConf = new KConfig(sConfFile);
}

